@@ -33,6 +33,22 @@ class AppState(object):
             service = s[0] if type(s) == list else s
             result.append(service)        
         self.services_active = result if result else self.services_active
+        
+    def validate_services(self,  saved_services):
+        ''' check loaded services '''
+        result = []
+        # check for new services
+        for s in self.services:
+            if  s not in saved_services:
+                saved_services.append(s)
+                self.services_active.append(s)
+        # check for removed services
+        for s in saved_services:
+            if  s not in self.services:
+                saved_services.remove(s)
+                if s in self.services_active:
+                    self.services_active.remove(s)
+        return saved_services
 
     def load(self):
         try:
@@ -41,13 +57,16 @@ class AppState(object):
             f.close()
             data = simplejson.loads(json)
             self.history = data.get('history',{})
-            self.services_active = data.get('services_active',self.services_active)            
+            self.services_active = data.get('services_active',self.services_active) 
+            #self.services = data.get('services',self.services)
+            saved_services = data.get('services',self.services)
+            self.services = self.validate_services(saved_services)
             self.normalize_active_services()
         except:
             print "no file found"
         
     def save(self):
-        state = {'history':self.history, 'services_active': self.services_active}
+        state = {'history':self.history, 'services_active': self.services_active, 'services':self.services}
         f = open(self.config_file,'w')
         simplejson.dump(state, f)
-        f.close()
\ No newline at end of file
+        f.close()
